﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using IdeaBlade.EntityModel;
using WinformsWalk.Model;
using System.Collections.ObjectModel;

namespace WinformsWalk {
  public partial class Form1 : Form {
    public Form1() {
      InitializeComponent();
      Products = new BindingList<Product>();
      Categories = new BindingList<Category>();
      Suppliers = new BindingList<Supplier>();
      Messages = new BindingList<string>();
    }

    private void Form1_Load(object sender, EventArgs e) {
      productsDataGrid.DataSource = Products;
      logListBox.DataSource = Messages;

      Log("Starting...");

      var query = _mgr.Products;
      var results = query.Execute();
      foreach (var item in results) {
        Products.Add(item);
      }

      foreach (var item in _mgr.Categories.Execute()) {
        Categories.Add(item);
      }

      foreach (var item in _mgr.Suppliers.Execute()) {
        Suppliers.Add(item);
      }

      productsDataGrid.Columns["ProductId"].Visible = false;
      productsDataGrid.Columns["SupplierID"].Visible = false;
      productsDataGrid.Columns["CategoryID"].Visible = false;
      productsDataGrid.Columns["QuantityPerUnit"].Visible = false;
      productsDataGrid.Columns["UnitsInStock"].Visible = false;
      productsDataGrid.Columns["UnitsOnOrder"].Visible = false;
      productsDataGrid.Columns["ReorderLevel"].Visible = false;
      productsDataGrid.Columns["Discontinued"].Visible = false;
      productsDataGrid.Columns["DiscontinuedDate"].Visible = false;
      productsDataGrid.Columns["RowVersion"].Visible = false;
      productsDataGrid.Columns["Category"].Visible = false;
      productsDataGrid.Columns["Supplier"].Visible = false;
      productsDataGrid.Columns["EntityAspect"].Visible = false;

      productsDataGrid.Columns["UnitPrice"].DefaultCellStyle.Format = "c";

      productNameTextBox.DataBindings.Add("Text", Products, "ProductName", true, DataSourceUpdateMode.OnPropertyChanged);
      productQuantityPerUnitTextBox.DataBindings.Add("Text", Products, "QuantityPerUnit", true, DataSourceUpdateMode.OnPropertyChanged);
      productPriceTextBox.DataBindings.Add("Text", Products, "UnitPrice", true, DataSourceUpdateMode.OnPropertyChanged);
      productUnitsInStockTextBox.DataBindings.Add("Text", Products, "UnitsInStock", true, DataSourceUpdateMode.OnPropertyChanged);
      productUnitsOnOrderTextBox.DataBindings.Add("Text", Products, "UnitsOnOrder", true, DataSourceUpdateMode.OnPropertyChanged);

      productPriceTextBox.DataBindings[0].FormatString = "c";

      categoryComboBox.DataSource = Categories;
      categoryComboBox.DisplayMember = Category.PathFor(c => c.CategoryName);
      categoryComboBox.ValueMember = Category.PathFor(c => c.Self);
      // Bind ComboBox.SelectedValue to the Product.Category entity
      categoryComboBox.DataBindings.Add("SelectedValue", Products, "Category");

      supplierComboBox.DataSource = Suppliers;
      supplierComboBox.DisplayMember = Supplier.PathFor(s => s.CompanyName);
      supplierComboBox.ValueMember = Supplier.PathFor(s => s.Self);
      // Bind ComboBox.SelectedValue to the Product.Supplier entity
      supplierComboBox.DataBindings.Add("SelectedValue", Products, "Supplier");

      productsDataGrid.CurrentCell = this.productsDataGrid[1, 0];
    }

    private void Log(string message) {
      message = String.Format("{0}: {1}", ++_messageCounter, message);
      Messages.Insert(0, message); // new messages on top
      logListBox.SelectedItem = message;
      logListBox.SelectedItem = null;
    }
    private int _messageCounter;

    private void btn_Save_Click(object sender, EventArgs e) {
      Log("Saving...");
      SaveResult saveResult;
      try {
        saveResult = _mgr.TrySaveChanges();
        if (saveResult.SaveStatus == SaveStatus.Normal) {
          Log("Successfully saved " + saveResult.SavedEntities.Count().ToString() + " entities.");
        } else if (saveResult.SaveStatus == SaveStatus.NoOperation) {
          Log("No entities to be saved.");
        } else {
          Log("Save failed.");
        }
      } catch (Exception exception) {
        Log("Save failed: " + exception.Message);
      }
    }

    private NorthwindIBEntities _mgr = new NorthwindIBEntities();
    private BindingList<Product> Products { get; set; }
    private BindingList<Category> Categories { get; set; }
    private BindingList<Supplier> Suppliers { get; set; }
    private BindingList<string> Messages { get; set; }
  }
}
