﻿Imports System.ComponentModel
Imports Model

Public Class Form1

    Public Sub New()
        InitializeComponent()
        Products = New BindingList(Of Product)()
        Categories = New BindingList(Of Category)()
        Suppliers = New BindingList(Of Supplier)()
        Messages = New BindingList(Of String)()
    End Sub



    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        productsDataGrid.DataSource = Products
        logListBox.DataSource = Messages

        Log("Starting...")

        ' Load Products
        For Each item As Product In _mgr.Products
            Products.Add(item)
        Next

        ' Load Categories
        For Each item As Category In _mgr.Categories.Execute()
            Categories.Add(item)
        Next

        ' Load Suppliers
        For Each item As Supplier In _mgr.Suppliers.Execute()
            Suppliers.Add(item)
        Next

        productsDataGrid.Columns("ProductId").Visible = False
        productsDataGrid.Columns("SupplierID").Visible = False
        productsDataGrid.Columns("CategoryID").Visible = False
        productsDataGrid.Columns("QuantityPerUnit").Visible = False
        productsDataGrid.Columns("UnitsInStock").Visible = False
        productsDataGrid.Columns("UnitsOnOrder").Visible = False
        productsDataGrid.Columns("ReorderLevel").Visible = False
        productsDataGrid.Columns("Discontinued").Visible = False
        productsDataGrid.Columns("DiscontinuedDate").Visible = False
        productsDataGrid.Columns("RowVersion").Visible = False
        productsDataGrid.Columns("Category").Visible = False
        productsDataGrid.Columns("Supplier").Visible = False
        productsDataGrid.Columns("EntityAspect").Visible = False

        productsDataGrid.Columns("UnitPrice").DefaultCellStyle.Format = "c"

        productNameTextBox.DataBindings.Add("Text", Products, "ProductName", True, DataSourceUpdateMode.OnPropertyChanged)
        productQuantityPerUnitTextBox.DataBindings.Add("Text", Products, "QuantityPerUnit", True, DataSourceUpdateMode.OnPropertyChanged)
        productPriceTextBox.DataBindings.Add("Text", Products, "UnitPrice", True, DataSourceUpdateMode.OnPropertyChanged)
        productUnitsInStockTextBox.DataBindings.Add("Text", Products, "UnitsInStock", True, DataSourceUpdateMode.OnPropertyChanged)
        productUnitsOnOrderTextBox.DataBindings.Add("Text", Products, "UnitsOnOrder", True, DataSourceUpdateMode.OnPropertyChanged)

        productPriceTextBox.DataBindings(0).FormatString = "c"

        categoryComboBox.DataSource = Categories
        categoryComboBox.DisplayMember = Category.PathFor(Function(c) c.CategoryName)
        categoryComboBox.ValueMember = Category.PathFor(Function(c) c.Self)
        ' Bind ComboBox.SelectedValue to the Product.Category entity
        categoryComboBox.DataBindings.Add("SelectedValue", Products, "Category")

        supplierComboBox.DataSource = Suppliers
        supplierComboBox.DisplayMember = Supplier.PathFor(Function(s) s.CompanyName)
        supplierComboBox.ValueMember = Supplier.PathFor(Function(s) s.Self)
        ' Bind ComboBox.SelectedValue to the Product.Supplier entity
        supplierComboBox.DataBindings.Add("SelectedValue", Products, "Supplier")

        productsDataGrid.CurrentCell = Me.productsDataGrid(1, 0)
    End Sub

    Private Sub Log(message As String)
        message = [String].Format("{0}: {1}", System.Threading.Interlocked.Increment(_messageCounter), message)
        Messages.Insert(0, message)
        ' new messages on top
        logListBox.SelectedItem = message
        logListBox.SelectedItem = Nothing
    End Sub

    Private Sub btn_Save_Click(sender As Object, e As EventArgs) Handles btn_Save.Click
        Log("Saving...")
        Dim saveResult As SaveResult
        Try
            saveResult = _mgr.TrySaveChanges()
            If saveResult.SaveStatus = SaveStatus.Normal Then
                Log("Successfully saved " + saveResult.SavedEntities.Count().ToString() + " entities.")
            ElseIf saveResult.SaveStatus = SaveStatus.NoOperation Then
                Log("No entities to be saved.")
            Else
                Log("Save failed.")
            End If
        Catch exception As Exception
            Log("Save failed: " + exception.Message)
        End Try
    End Sub


    Private ReadOnly _mgr As New NorthwindIBEntities()
    Private Property Products() As BindingList(Of Product)
    Private Property Categories() As BindingList(Of Category)
    Private Property Suppliers() As BindingList(Of Supplier)
    Private Property Messages() As BindingList(Of String)
    Private _messageCounter As Integer

End Class
